@@ -2,18 +2,31 @@ package Text::CSV::BulkData;
 
 use strict;
 use vars qw($VERSION);
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Carp;
 
 sub new { 
     my ($class, $output_file, $format) = @_;
     bless { 
-        output_file => $output_file,
-        format => $format,
+        output_file   => $output_file,
+        format        => $format,
+        residue_loop  => 1,
     }, $class;
 } 
 
+sub set_residue_loop_off {
+    my ($self, $output_file) = @_;
+    $self->{residue_loop} = 0;
+    return $self;
+}
+
+sub set_output_file {
+    my ($self, $output_file) = @_;
+    $self->{output_file} = $output_file;
+    return $self;
+}
+
 sub set_format {
     my ($self, $format) = @_;
     $self->{format} = $format;
@@ -81,7 +94,9 @@ sub _calculate {
 
     if ( $pattern =~ m{\*(\d+)} ) {
         if ( $flag eq 1) {
-            $pattern = $self->_return_substituted('^\d+\*\d+', $self->{before} * $1);
+            my $res = $self->{before} * $1;
+            $res = $self->_recursive_calc($res);
+            $pattern = $self->_return_substituted('^\d+\*\d+', $res);
         } else { 
             my $res = $i * $1;
             $pattern =~ s{\*\d+}{$res};
@@ -89,15 +104,20 @@ sub _calculate {
         $self->_calculate($pattern, $i, 0);
     } elsif ( $pattern =~ m{/(\d+)} ) {
         if ( $flag eq 1) {
-            $pattern = $self->_return_substituted('^\d+/\d+', int($self->{before} / $1));
+            my $res = int($self->{before} / $1);
+            $res = $self->_recursive_calc($res);
+            $pattern = $self->_return_substituted('^\d+/\d+', $res);
         } else { 
             my $res = int($i / $1);
             $pattern =~ s{/\d+}{$res};
         }
         $self->_calculate($pattern, $i, 0);
     } elsif ( $pattern =~ m{%(\d+)} ) {
+        $self->{residue_loop} = $1 if $self->{residue_loop};
         if ( $flag eq 1) {
-            $pattern = $self->_return_substituted('^\d+%\d+', $self->{before} % $1);
+            my $res = $self->{before} % $1;
+            $res = $self->_recursive_calc($res);
+            $pattern = $self->_return_substituted('^\d+%\d+', $res);
         } else { 
             my $res = $i % $1;
             $pattern =~ s{%\d+}{$res};
@@ -105,7 +125,9 @@ sub _calculate {
         $self->_calculate($pattern, $i, 0);
     } elsif ( $pattern =~ m{\+(\d+)} ) {
         if ( $flag eq 1) {
-            $pattern = $self->_return_substituted('^\d+\+\d+', $self->{before} + $1);
+            my $res = $self->{before} + $1;
+            $res = $self->_recursive_calc($res);
+            $pattern = $self->_return_substituted('^\d+\+\d+', $res);
         } else { 
             my $res = $i + $1;
             $pattern =~ s{\+\d+}{$res};
@@ -113,17 +135,32 @@ sub _calculate {
         $self->_calculate($pattern, $i, 0);
     } elsif ( $pattern =~ m{-(\d+)} ) {
         if ( $flag eq 1) {
-            $pattern = $self->_return_substituted('^\d+-\d+', $self->{before} - $1);
+            my $res = $self->{before} - $1;
+            $res = $self->_recursive_calc($res);
+            $pattern = $self->_return_substituted('^\d+-\d+', $res);
         } else { 
             my $res = $i - $1;
             $pattern =~ s{-\d+}{$res};
         }
         $self->_calculate($pattern, $i, 0);
     } else {
+        delete $self->{pattern}, $self->{before};
+        $self->{residue_loop} = 1;
         return $pattern;
     }
 }
 
+sub _recursive_calc {
+    my ($self, $res) = @_;
+    if ( defined $self->{residue_loop} 
+         && $self->{residue_loop} > 1
+         && ($res < 0 || $res > $self->{residue_loop})
+       ) {
+        $res += $self->{residue_loop};
+    }
+    return $res;
+}
+
 sub _return_substituted {
     my ($self, $regexp, $res) = @_;
     $self->{pattern} =~ s{$regexp}{$res};
@@ -153,31 +190,44 @@ Text::CSV::BulkData - generate csv file with bulk data
 
   use Text::CSV::BulkData;
 
-  my $output_file = "/your/dir/example.dat";
-  my $format = "0907000%04d,JPN,160-%04d,type000%04d,0120444%04d,20080418100000\n";
+  my $output_file_1 = "/your/dir/example.dat";
+  my $format_1 = "0907000%04d,JPN,160-%04d,type000%04d,0120444%04d,2008041810%02d00,2008041811%02d00\n";
+  my $pattern_1 = [undef,'*2*2','-2','*2+1','%60-1','%60-40'];
 
-  my $gen = Text::CSV::BulkData->new($output_file, $format);
+  my $gen = Text::CSV::BulkData->new($output_file_1, $format);
 
   my $pattern_1 = [undef,'*2','-2','*2+1'];
   $gen->initialize
       ->set_pattern($pattern_1)
-      ->set_start(1)
-      ->set_end(3)
+      ->set_start(59)
+      ->set_end(62)
       ->make;
-  my $pattern_2 = [undef,'/10','*3/2','%2'];
-  $gen->set_pattern($pattern_2)
-      ->set_start(8)
-      ->set_end(10)
+
+  my $output_file_2 = "/your/dir/yetanotherfile.dat";
+  my $format_2  = "0907000%04d,JPN,160-%04d,type000%04d,0120444%04d,20080418%02d0000,20080419%02d0000\n";
+  my $pattern_2 = [undef,'/10','*3/2','%2', '%24-1','%24'];
+
+  $gen->set_output_file($output_file_2)
+      ->set_format($format_2)
+      ->set_pattern($pattern_2)
+      ->set_start(239)
+      ->set_end(241)
       ->make;
 
 This sample generates following csv file.
 
-  09070000001,JPN,160-0002,type0000000,01204440003,20080418100000
-  09070000002,JPN,160-0004,type0000000,01204440005,20080418100000
-  09070000003,JPN,160-0006,type0000001,01204440007,20080418100000
-  09070000008,JPN,160-0000,type0000012,01204440000,20080418100000
-  09070000009,JPN,160-0000,type0000013,01204440001,20080418100000
-  09070000010,JPN,160-0001,type0000015,01204440000,20080418100000
+/your/dir/example.dat
+
+  09070000059,JPN,160-0236,type0000057,01204440119,20080418105800,20080418111900
+  09070000060,JPN,160-0240,type0000058,01204440121,20080418105900,20080418112000
+  09070000061,JPN,160-0244,type0000059,01204440123,20080418100000,20080418112100
+  09070000062,JPN,160-0248,type0000060,01204440125,20080418100100,20080418112200
+
+/your/dir/yetanotherfile.dat
+
+  09070000239,JPN,160-0023,type0000358,01204440001,20080418220000,20080419230000
+  09070000240,JPN,160-0024,type0000360,01204440000,20080418230000,20080419000000
+  09070000241,JPN,160-0024,type0000361,01204440001,20080418000000,20080419010000
 
 =head1 DESCRIPTION
 
